unit mainform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, FtpSynchronize, ComCtrls, Inifiles;

type
  TForm1 = class(TForm)
    GroupBox1: TGroupBox;
    edtLocalPath: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    edtHostname: TEdit;
    label3: TLabel;
    edtHostPort: TEdit;
    edtRemotePath: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    edtUserName: TEdit;
    edtPassword: TEdit;
    label6: TLabel;
    mmoLog: TMemo;
    btnStart: TButton;
    StatusBar: TStatusBar;
    btnSave: TButton;
    Label7: TLabel;
    memoExclude: TMemo;
    cbxCaseInsensitive: TCheckBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnStartClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FtpSync: TFtpSync;
    DeleteAll: Boolean;
    Ini: TIniFile;
    procedure FtpSyncBeforeCommand(Sender: TFtpSync; Command: TFtpSyncCommand; const CommandParameter, StatusMessage: String);
    procedure FtpSyncAfterCommand(Sender: TFtpSync; Command: TFtpSyncCommand; const CommandParameter, StatusMessage: String);
    procedure FtpSyncError(Sender: TFtpSync; Command: TFtpSyncCommand; const CommandParameter, ErrorMessage: String);
    procedure FtpSyncFinished(Sender: TFtpSync; Success: Boolean);
    procedure FtpSyncConfirmDelete(Sender: TFtpSync; const RemotePath, RemoteFile: String; Directory: Boolean; var AllowDelete: Boolean);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Assigned(FtpSync) then
  begin
    FtpSync.Terminate;
    Sleep(1000);
    FtpSync.Free;
  end;
end;

procedure TForm1.btnStartClick(Sender: TObject);
begin
  if Assigned(FtpSync) then
  begin
    FtpSync.Abort;
    FtpSync.Free;
  end;
  FtpSync := TFtpSync.Create;
  FtpSync.OnBeforeCommand := FtpSyncBeforeCommand;
  FtpSync.OnAfterCommand := FtpSyncAfterCommand;
  FtpSync.OnError := FtpSyncError;
  FtpSync.OnFinished := FtpSyncFinished;
  FtpSync.OnConfirmDelete := FtpSyncConfirmDelete;

  FtpSync.LocalPath := edtLocalPath.Text;
  FtpSync.Host := edtHostname.Text;
  FtpSync.Port := StrToInt(edtHostPort.Text);
  FtpSync.AccountName := edtUserName.Text;
  FtpSync.Password := edtPassword.Text;
  FtpSync.RemotePath := edtRemotePath.Text;
  FtpSync.Options := [soRecurse];
  FtpSync.ExcludeFiles.Text := memoExclude.Lines.Text;

  if cbxCaseInsensitive.Checked then
    FtpSync.Options := FtpSync.Options + [soCaseInsensitive]
  else
    FtpSync.Options := FtpSync.Options - [soCaseInsensitive];
  
  FtpSync.Resume;
end;

procedure TForm1.FtpSyncAfterCommand(Sender: TFtpSync; Command: TFtpSyncCommand; const CommandParameter, StatusMessage: String);
begin
//  mmoLog.Lines.Add(StatusMessage);
  StatusBar.SimpleText := StatusMessage;
end;

procedure TForm1.FtpSyncBeforeCommand(Sender: TFtpSync; Command: TFtpSyncCommand; const CommandParameter, StatusMessage: String);
begin
  while mmoLog.Lines.Count > 300 do
    mmoLog.Lines.Delete(0);

  mmoLog.Lines.Add(StatusMessage);
  StatusBar.SimpleText := StatusMessage;
end;

procedure TForm1.FtpSyncError(Sender: TFtpSync; Command: TFtpSyncCommand;
  const CommandParameter, ErrorMessage: String);
begin
  mmoLog.Lines.Add(ErrorMessage);
  StatusBar.SimpleText := ErrorMessage;
end;

procedure TForm1.FtpSyncFinished(Sender: TFtpSync; Success: Boolean);
begin
  if Success then
    StatusBar.SimpleText := '*** Done, no errors. ***'
  else
    StatusBar.SimpleText := '*** Aborted to due errors ***';
  mmoLog.Lines.Add(StatusBar.SimpleText);
end;

procedure TForm1.FtpSyncConfirmDelete(Sender: TFtpSync; const RemotePath, RemoteFile: String; Directory: Boolean; var AllowDelete: Boolean);
begin
  DeleteAll := True;
  
  if DeleteAll then
  begin
    AllowDelete := True;
    Exit;
  end
  else begin
    case MessageDlg('Delete ' + RemotePath + '/' + RemoteFile +' ?', mtConfirmation, [mbYes, mbNo, mbAll], 0) of
      mrYes: AllowDelete := True;
      mrNo: AllowDelete := False;
      mrAll: begin
        AllowDelete := True;
        DeleteAll := True;
      end;
    end;
  end;
end;

procedure TForm1.btnSaveClick(Sender: TObject);
var
  S: String;
begin
  Ini.WriteString('cfg', 'localpath', edtLocalPath.Text);
  Ini.WriteString('cfg', 'hostname', edtHostname.Text);
  Ini.WriteString('cfg', 'hostport', edtHostport.Text);
  Ini.WriteString('cfg', 'username', edtUserName.Text);
  Ini.WriteString('cfg', 'password', edtPassword.Text);
  Ini.WriteString('cfg', 'remotepath', edtRemotePath.Text);
  Ini.WriteBool('cfg', 'caseinsensitive', cbxCaseInsensitive.Checked);

  S := memoExclude.Text;
  Ini.WriteString('cfg', 'exclude', StringReplace(S, #13#10, ';', [rfReplaceAll]));
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  S: String;
begin
  S := IncludeTrailingBackslash(ExtractFilePath(Application.ExeName)) + 'ftpsync.ini';
  Ini := TIniFile.Create(S);

  edtLocalPath.Text := Ini.ReadString('cfg', 'localpath', '');
  edtHostname.Text := Ini.ReadString('cfg', 'hostname', '');
  edtHostport.Text := Ini.ReadString('cfg', 'hostport', '21');
  edtUserName.Text := Ini.ReadString('cfg', 'username', '');
  edtPassword.Text := Ini.ReadString('cfg', 'password', '');
  edtRemotePath.Text := Ini.ReadString('cfg', 'remotepath', '');
  cbxCaseInsensitive.Checked := Ini.ReadBool('cfg', 'caseinsensitive', False);
  S := Ini.ReadString('cfg', 'exclude', '');
  memoExclude.Text := StringReplace(S, ';', #13#10, [rfReplaceAll]);
end;

end.
